<?php

declare(strict_types=1);

namespace App\Domain\Registry\Symfony\EventSubscriber\Doctrine;

use App\Domain\Registry\Calculator\ConformiteOrganisationConformiteCalculator;
use App\Domain\Registry\Model\ConformiteOrganisation\Evaluation;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\Event\PrePersistEventArgs;
use Doctrine\ORM\Event\PreUpdateEventArgs;

class ConformiteOrganisationSubscriber implements EventSubscriber
{
    /**
     * @var ConformiteOrganisationConformiteCalculator
     */
    private $conformiteCalculator;

    public function __construct(ConformiteOrganisationConformiteCalculator $calculator)
    {
        $this->conformiteCalculator = $calculator;
    }

    public function getSubscribedEvents(): array
    {
        return [
            'prePersist',
            'preUpdate',
        ];
    }

    public function prePersist(PrePersistEventArgs $args)
    {
        $this->processConformiteOrganisation($args);
    }

    public function preUpdate(PreUpdateEventArgs $args)
    {
        $this->processConformiteOrganisation($args);
    }

    private function processConformiteOrganisation(PrePersistEventArgs|PreUpdateEventArgs $args)
    {
        if (!($evaluation = $args->getObject()) instanceof Evaluation) {
            return;
        }

        /* @var Evaluation $evaluation */
        $this->conformiteCalculator->calculEvaluationConformites($evaluation);
    }
}
