<?php

/**
 * This file is part of the MADIS - RGPD Management application.
 *
 * @copyright Copyright (c) 2018-2019 Soluris - Solutions Numériques Territoriales Innovantes
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

declare(strict_types=1);

namespace App\Tests\Domain\Registry\Symfony\EventSubscriber\Doctrine;

use App\Domain\Registry\Calculator\Completion\ConformiteTraitementCompletion;
use App\Domain\Registry\Dictionary\MesurementStatusDictionary;
use App\Domain\Registry\Model\ConformiteTraitement\Reponse;
use App\Domain\Registry\Model\Mesurement;
use App\Domain\Registry\Symfony\EventSubscriber\Doctrine\MesurementSubscriber;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\Event\PostUpdateEventArgs;
use Doctrine\Persistence\ObjectManager;
use PHPUnit\Framework\TestCase;
use Prophecy\PhpUnit\ProphecyTrait;
use Prophecy\Prophecy\ObjectProphecy;

class MesurementSubscriberTest extends TestCase
{
    use ProphecyTrait;

    /**
     * @var ConformiteTraitementCompletion
     */
    private $conformiteTraitementCompletion;

    /**
     * @var MesurementSubscriber
     */
    private $subscriber;

    private ObjectProphecy|EntityManagerInterface $entityManager;

    public function setUp(): void
    {
        $this->subscriber    = new MesurementSubscriber();
        $this->entityManager = $this->prophesize(EntityManagerInterface::class);
    }

    /**
     * Test instance of Subscriber.
     */
    public function testInstanceOf()
    {
        $this->assertInstanceOf(EventSubscriber::class, $this->subscriber);
    }

    /**
     * Test getSubscribedEvents of current subscriber.
     */
    public function testGetSubscribedEvents()
    {
        $this->assertEquals(
            [
                'postUpdate',
            ],
            $this->subscriber->getSubscribedEvents()
        );
    }

    public function testPostUpdateWithAppliedStatus()
    {
        $object  = $this->prophesize(Mesurement::class);
        $reponse = $this->prophesize(Reponse::class);

        $object->getStatus()->shouldBeCalled()->willReturn(MesurementStatusDictionary::STATUS_APPLIED);
        $object->getConformiteTraitementReponses()->shouldBeCalled()->willReturn([$reponse->reveal()]);
        $this->entityManager->persist($reponse->reveal())->shouldBeCalled();
        $this->entityManager->flush()->shouldBeCalled();

        $eventArgs = new PostUpdateEventArgs($object->reveal(), $this->entityManager->reveal());

        $reponse->addActionProtectionsPlanifiedNotSeen($object->reveal())->shouldBeCalled();
        $reponse->removeActionProtection($object->reveal())->shouldBeCalled();

        $this->subscriber->postUpdate($eventArgs);
    }

    public function testPostUpdateWithNotAppliedStatus()
    {
        $object        = $this->prophesize(Mesurement::class);
        $reponse       = $this->prophesize(Reponse::class);
        $objectManager = $this->prophesize(ObjectManager::class);

        $object->getStatus()->shouldBeCalled()->willReturn('foo');
        $object->getConformiteTraitementReponses()->shouldBeCalled()->willReturn([$reponse->reveal()]);
        $this->entityManager->persist($reponse->reveal())->shouldBeCalled();
        $this->entityManager->flush()->shouldBeCalled();

        $eventArgs = new PostUpdateEventArgs($object->reveal(), $this->entityManager->reveal());

        $reponse->removeActionProtectionsPlanifiedNotSeen($object->reveal())->shouldBeCalled();

        $this->subscriber->postUpdate($eventArgs);
    }
}
